import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable, Subject } from 'rxjs';
import { LoginModel } from '../../models/login.model';
import { RegisterModel } from '../../models/register.model';
import { TokenResponse } from '../../models/token.response';
import { UserModel } from '../../models/user.model';
import { AuthTokenService } from '../../../shared/services/auth/auth-token.service';
import { Router } from '@angular/router';

@Injectable()
export class AuthService {

    private static updateAgent: any;
    private user: UserModel;
    public userChange = new Subject<UserModel>();

    constructor(
        private http: HttpClient,
        private authTokenService: AuthTokenService,
        private router: Router,
    ) {
        this.checkUser();
        this.userChange.subscribe(user => {
            this.user = user;
        });
    }

    getUser(): UserModel {
        return this.user;
    }

    checkUser() {
        this.http.post<UserModel>('/api/user-check', {}).subscribe(user => {
            this.userChange.next(user);
            if (AuthService.updateAgent) {
                clearTimeout(AuthService.updateAgent);
            }
            AuthService.updateAgent = setTimeout(() => {
                this.checkUser();
            }, 2000) as any;
        }, () => {
            this.logout();
        });
    }

    createAccount(data: RegisterModel): Observable<UserModel> {
        return this.http.post<UserModel>('/api/register', data);
    }

    login(data: LoginModel): Observable<TokenResponse> {
        const out = new Subject<TokenResponse>();
        this.http.post<TokenResponse>('/api/login', {
            username: data.email,
            password: data.password,
        }).subscribe(data => {
            this.authTokenService.setToken(data.token);
            this.checkUser();
            out.next(data);
        });
        return out;
    }

    logout() {
        this.authTokenService.removeToken();
        this.router.navigate(['/auth']);
    }
}
